.conditions-field {
  overflow: hidden;
}
.conditions-field .conditions-field-action {
  cursor: pointer;
  float: left;
  display: inline-block;
  margin-right: 0.3em;
  text-align: center;
  width: 20px;
  height: 20px;
}
.conditions-field .conditions-field-action .fa {
  line-height: 20px;
  font-size: 16px;
}
.conditions-field .conditions-field-add-condition {
  background: #F9F9F9;
  clear: both;
  display: inline-block;
  line-height: 20px;
  padding: 0.5em;
  color: #222222;
}
.conditions-field .conditions-field-add-condition .fa {
  color: #00AA00;
}
.conditions-field .conditions-field-choice {
  margin-right: 0.3em;
}
.conditions-field .conditions-field-choice select {
  min-width: 130px;
}
.conditions-field .conditions-field-mode ul {
  margin: 0 0 1em 0;
  padding: 0;
}
.conditions-field .conditions-field-mode ul li {
  display: inline-block;
  margin-left: 1em;
}
.conditions-field .conditions-field-mode ul li input {
  margin: 0;
  vertical-align: baseline;
}
.conditions-field .conditions-field-mode ul li label {
  display: inline-block;
}
.conditions-field .conditions-field-operator {
  margin-right: 0.3em;
}
.conditions-field .conditions-field-operator select {
  min-width: 130px;
}
.conditions-field .conditions-field-rows-container {
  border: 1px #CCC solid;
  border-radius: 6px;
  overflow: hidden;
  /* Don't let backgrounds overlap borders. */
}
.conditions-field .conditions-field-rows {
  background: #F3F3F3;
  margin: 0;
  padding: 0;
}
.conditions-field .conditions-field-row {
  border-bottom: 1px #CCC solid;
  clear: both;
  margin: 0;
  padding: 0.5em;
}
.conditions-field .conditions-field-row select {
  vertical-align: top;
}
.conditions-field .conditions-field-row-delete .fa {
  color: #CC0000;
}
.conditions-field .conditions-field-row-options {
  display: block;
  overflow: hidden;
}
.conditions-field .error-list {
  color: #990000;
  margin: 0 0 0.5em 0;
  padding: 0;
}
/*# sourceMappingURL=conditions.css.map */