from __future__ import unicode_literals


settings_map = {
    'recaptcha_private_key': 'RECAPTCHA_PRIVATE_KEY',
    'recaptcha_public_key': 'RECAPTCHA_PUBLIC_KEY',
}


defaults = {
    'recaptcha_private_key': None,
    'recaptcha_public_key': None,
}
