{% load djblets_utils %}
<link href="{{url}}" rel="stylesheet{% if url|endswith:"less" %}/less{% endif %}" type="text/css"{% if media %} media="{{media}}"{% endif %}{% if title %} title="{{title|default:"all"}}"{% endif %}{% if charset %} charset="{{charset}}"{% endif %} />
